@@ -19,12 +19,17 @@ var analyzeCmd = &cobra.Command{
 	Args:   cobra.NoArgs,
 	RunE: func(cmd *cobra.Command, _ []string) error {
 		ctx := cmd.Context()
-		state, err := state.New(ctx, flags.PostgresURL(), flags.StateSchema())
+		state, err := state.New(ctx, flags.PostgresURL(), flags.StateSchema(), state.WithPgrollVersion(Version))
 		if err != nil {
 			return err
 		}
 		defer state.Close()
 
+		// Ensure that pgroll is initialized
+		if err := EnsureInitialized(ctx, state); err != nil {
+			return err
+		}
+
 		schema, err := state.ReadSchema(ctx, flags.Schema())
 		if err != nil {
 			return err
