@@ -13,7 +13,8 @@ var completeCmd = &cobra.Command{
 	Use:   "complete <file>",
 	Short: "Complete an ongoing migration with the operations present in the given file",
 	RunE: func(cmd *cobra.Command, args []string) error {
-		m, err := NewRoll(cmd.Context())
+		// Create a roll instance and check if pgroll is initialized
+		m, err := NewRollWithInitCheck(cmd.Context())
 		if err != nil {
 			return err
 		}
