@@ -27,3 +27,9 @@ func SkipValidation() bool { return viper.GetBool("SKIP_VALIDATION") }
 func Role() string {
 	return viper.GetString("ROLE")
 }
+
+func Verbose() bool { return viper.GetBool("VERBOSE") }
+
+func UseVersionSchema() bool {
+	return viper.GetBool("USE_VERSION_SCHEMA")
+}
