@@ -10,10 +10,11 @@ import (
 )
 
 var rollbackCmd = &cobra.Command{
-	Use:   "rollback <file>",
+	Use:   "rollback",
 	Short: "Roll back an ongoing migration",
 	RunE: func(cmd *cobra.Command, args []string) error {
-		m, err := NewRoll(cmd.Context())
+		// Create a roll instance and check if pgroll is initialized
+		m, err := NewRollWithInitCheck(cmd.Context())
 		if err != nil {
 			return err
 		}
