@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"math"
 	"os"
+	"path/filepath"
 	"time"
 
 	"github.com/pterm/pterm"
@@ -30,20 +31,32 @@ func startCmd() *cobra.Command {
 		Args:      cobra.ExactArgs(1),
 		ValidArgs: []string{"file"},
 		RunE: func(cmd *cobra.Command, args []string) error {
+			ctx := cmd.Context()
 			fileName := args[0]
 
-			m, err := NewRoll(cmd.Context())
+			// Create a roll instance and check if pgroll is initialized
+			m, err := NewRollWithInitCheck(ctx)
 			if err != nil {
 				return err
 			}
 			defer m.Close()
 
+			// Check whether the schema needs an initial baseline migration
+			needsBaseline, err := m.State().HasExistingSchemaWithoutHistory(ctx, m.Schema())
+			if err != nil {
+				return fmt.Errorf("failed to check for existing schema: %w", err)
+			}
+			if needsBaseline {
+				fmt.Printf("Schema %q is non-empty but has no migration history. Run `pgroll baseline` first\n", m.Schema())
+				return nil
+			}
+
 			c := backfill.NewConfig(
 				backfill.WithBatchSize(batchSize),
 				backfill.WithBatchDelay(batchDelay),
 			)
 
-			return runMigrationFromFile(cmd.Context(), m, fileName, complete, c)
+			return runMigrationFromFile(ctx, m, fileName, complete, c)
 		},
 	}
 
@@ -58,7 +71,7 @@ func startCmd() *cobra.Command {
 }
 
 func runMigrationFromFile(ctx context.Context, m *roll.Roll, fileName string, complete bool, c *backfill.Config) error {
-	migration, err := readMigration(fileName)
+	migration, err := migrations.ReadMigration(os.DirFS(filepath.Dir(fileName)), filepath.Base(fileName))
 	if err != nil {
 		return err
 	}
@@ -92,25 +105,15 @@ func runMigration(ctx context.Context, m *roll.Roll, migration *migrations.Migra
 		}
 	}
 
-	version := migration.Name
-	viewName := roll.VersionedSchemaName(flags.Schema(), version)
-	msg := fmt.Sprintf("New version of the schema available under the postgres %q schema", viewName)
+	var msg string
+	if m.UseVersionSchema() {
+		viewName := roll.VersionedSchemaName(flags.Schema(), migration.VersionSchemaName())
+		msg = fmt.Sprintf("New version of the schema available under the postgres %q schema", viewName)
+	} else {
+		msg = fmt.Sprintf("Migration %q started successfully", migration.Name)
+	}
+
 	sp.Success(msg)
 
 	return nil
 }
-
-func readMigration(fileName string) (*migrations.Migration, error) {
-	file, err := os.Open(fileName)
-	if err != nil {
-		return nil, fmt.Errorf("opening migration file: %w", err)
-	}
-	defer file.Close()
-
-	migration, err := migrations.ReadMigration(file)
-	if err != nil {
-		return nil, fmt.Errorf("reading migration file: %w", err)
-	}
-
-	return migration, nil
-}
