@@ -41,6 +41,16 @@ type ColumnAlreadyExistsError struct {
 	Name  string
 }
 
+type ColumnIsInvalidError struct {
+	Table string
+	Name  string
+}
+
+func (e ColumnIsInvalidError) Error() string {
+	return fmt.Sprintf(`column %q in table %q is missing one or more required fields - "name" or "type"`,
+		e.Name, e.Table)
+}
+
 func (e ColumnAlreadyExistsError) Error() string {
 	return fmt.Sprintf("column %q already exists in table %q", e.Name, e.Table)
 }
@@ -134,6 +144,7 @@ func (e ColumnReferenceError) Error() string {
 type CheckConstraintError struct {
 	Table  string
 	Column string
+	Name   string
 	Err    error
 }
 
@@ -142,6 +153,13 @@ func (e CheckConstraintError) Unwrap() error {
 }
 
 func (e CheckConstraintError) Error() string {
+	if e.Column == "" {
+		return fmt.Sprintf("check constraint %q in table %q is invalid: %s",
+			e.Name,
+			e.Table,
+			e.Err.Error())
+	}
+
 	return fmt.Sprintf("check constraint on column %q in table %q is invalid: %s",
 		e.Table,
 		e.Column,
@@ -166,26 +184,6 @@ func (e ConstraintAlreadyExistsError) Error() string {
 	return fmt.Sprintf("constraint %q on table %q already exists", e.Constraint, e.Table)
 }
 
-type NoUpSQLAllowedError struct{}
-
-func (e NoUpSQLAllowedError) Error() string {
-	return "up SQL is not allowed for this operation"
-}
-
-type NoDownSQLAllowedError struct{}
-
-func (e NoDownSQLAllowedError) Error() string {
-	return "down SQL is not allowed for this operation"
-}
-
-type BackfillNotPossibleError struct {
-	Table string
-}
-
-func (e BackfillNotPossibleError) Error() string {
-	return fmt.Sprintf("a backfill is required but table %q doesn't have a single column primary key or a UNIQUE, NOT NULL column", e.Table)
-}
-
 type InvalidReplicaIdentityError struct {
 	Table    string
 	Identity string
@@ -203,15 +201,39 @@ type InvalidOnDeleteSettingError struct {
 func (e InvalidOnDeleteSettingError) Error() string {
 	return fmt.Sprintf("foreign key %q on_delete setting must be one of: %q, %q, %q, %q or %q, not %q",
 		e.Name,
-		ForeignKeyReferenceOnDeleteNOACTION,
-		ForeignKeyReferenceOnDeleteRESTRICT,
-		ForeignKeyReferenceOnDeleteSETDEFAULT,
-		ForeignKeyReferenceOnDeleteSETNULL,
-		ForeignKeyReferenceOnDeleteCASCADE,
+		ForeignKeyActionNOACTION,
+		ForeignKeyActionRESTRICT,
+		ForeignKeyActionSETDEFAULT,
+		ForeignKeyActionSETNULL,
+		ForeignKeyActionCASCADE,
 		e.Setting,
 	)
 }
 
+type UnexpectedOnDeleteSetColumnError struct {
+	Name string
+}
+
+func (e UnexpectedOnDeleteSetColumnError) Error() string {
+	return fmt.Sprintf("if on_delete_set_columns is set in foreign key %q, on_delete setting must be one of: %q, %q",
+		e.Name,
+		ForeignKeyActionSETDEFAULT,
+		ForeignKeyActionSETNULL,
+	)
+}
+
+type InvalidOnDeleteSetColumnError struct {
+	Name   string
+	Column string
+}
+
+func (e InvalidOnDeleteSetColumnError) Error() string {
+	return fmt.Sprintf("invalid on_delete_set_columns setting: column %q is not part of the foreign key %q",
+		e.Column,
+		e.Name,
+	)
+}
+
 type AlterColumnNoChangesError struct {
 	Table  string
 	Column string
@@ -250,3 +272,28 @@ type MultiColumnConstraintsNotSupportedError struct {
 func (e MultiColumnConstraintsNotSupportedError) Error() string {
 	return fmt.Sprintf("constraint %q on table %q applies to multiple columns", e.Constraint, e.Table)
 }
+
+type PrimaryKeysAreAlreadySetError struct {
+	Table string
+}
+
+func (e PrimaryKeysAreAlreadySetError) Error() string {
+	return fmt.Sprintf("table %q already has a primary key configuration in columns list", e.Table)
+}
+
+type InvalidGeneratedColumnError struct {
+	Table  string
+	Column string
+}
+
+func (e InvalidGeneratedColumnError) Error() string {
+	return fmt.Sprintf("column %q on table %q is invalid: only one of generated.expression and generated.identity may be set", e.Column, e.Table)
+}
+
+type UpSQLMustBeColumnDefaultError struct {
+	Column string
+}
+
+func (e UpSQLMustBeColumnDefaultError) Error() string {
+	return fmt.Sprintf(`volatile default expression for column %q; "up" must be equal to "default"`, e.Column)
+}
