@@ -4,35 +4,53 @@ package migrations
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 
 	_ "github.com/lib/pq"
 
+	"github.com/xataio/pgroll/pkg/backfill"
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
 
 // Operation is an operation that can be applied to a schema
 type Operation interface {
-	// Start will apply the required changes to enable supporting the new schema
+	// Start will return the list of required changes to enable supporting the new schema
 	// version in the database (through a view)
 	// update the given views to expose the new schema version
 	// Returns the table that requires backfilling, if any.
-	Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error)
+	Start(ctx context.Context, l Logger, conn db.DB, s *schema.Schema) (*StartResult, error)
 
 	// Complete will update the database schema to match the current version
 	// after calling Start.
 	// This method should be called once the previous version is no longer used.
-	Complete(ctx context.Context, conn db.DB, s *schema.Schema) error
+	Complete(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error)
 
 	// Rollback will revert the changes made by Start. It is not possible to
 	// rollback a completed migration.
-	Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error
+	Rollback(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error)
 
 	// Validate returns a descriptive error if the operation cannot be applied to the given schema.
 	Validate(ctx context.Context, s *schema.Schema) error
 }
 
+// Createable interface must be implemented for all operations
+// that can be created using the CLI create command.
+//
+// The function must prompt users to configure all attributes of an operation.
+//
+// Example implementation for OpMyOperation that has 3 attributes: table, column and down:
+//
+//	func (o *OpMyOperation) Create() {
+//		o.Table, _ = pterm.DefaultInteractiveTextInput.WithDefaultText("table").Show()
+//		o.Column, _ = pterm.DefaultInteractiveTextInput.WithDefaultText("column").Show()
+//		o.Down, _ = pterm.DefaultInteractiveTextInput.WithDefaultText("down").Show()
+//	}
+type Createable interface {
+	Create()
+}
+
 // IsolatedOperation is an operation that cannot be executed with other operations
 // in the same migration.
 type IsolatedOperation interface {
@@ -51,12 +69,31 @@ type RequiresSchemaRefreshOperation interface {
 type (
 	Operations []Operation
 	Migration  struct {
-		Name string `json:"name"`
+		Name          string     `json:"-"`
+		VersionSchema string     `json:"version_schema,omitempty"`
+		Operations    Operations `json:"operations"`
+	}
+	RawMigration struct {
+		Name          string          `json:"-"`
+		VersionSchema string          `json:"version_schema,omitempty"`
+		Operations    json.RawMessage `json:"operations"`
+	}
 
-		Operations Operations `json:"operations"`
+	StartResult struct {
+		Actions      []DBAction
+		BackfillTask *backfill.Task
 	}
 )
 
+// VersionSchemaName returns the version schema name for the migration.
+// It defaults to the migration name if no version schema is set.
+func (m *Migration) VersionSchemaName() string {
+	if m.VersionSchema != "" {
+		return m.VersionSchema
+	}
+	return m.Name
+}
+
 // Validate will check that the migration can be applied to the given schema
 // returns a descriptive error if the migration is invalid
 func (m *Migration) Validate(ctx context.Context, s *schema.Schema) error {
@@ -86,19 +123,9 @@ func (m *Migration) UpdateVirtualSchema(ctx context.Context, s *schema.Schema) e
 	// Run `Start` on each operation using the fake DB. Updates will be made to
 	// the in-memory schema `s` without touching the physical database.
 	for _, op := range m.Operations {
-		if _, err := op.Start(ctx, db, "", s); err != nil {
+		if _, err := op.Start(ctx, NewNoopLogger(), db, s); err != nil {
 			return err
 		}
 	}
 	return nil
 }
-
-// ContainsRawSQLOperation returns true if the migration contains a raw SQL operation
-func (m *Migration) ContainsRawSQLOperation() bool {
-	for _, op := range m.Operations {
-		if _, ok := op.(*OpRawSQL); ok {
-			return true
-		}
-	}
-	return false
-}
