@@ -22,7 +22,8 @@ func TestAddColumn(t *testing.T) {
 			name: "add column",
 			migrations: []migrations.Migration{
 				{
-					Name: "01_add_table",
+					Name:          "01_add_table",
+					VersionSchema: "add_table",
 					Operations: migrations.Operations{
 						&migrations.OpCreateTable{
 							Name: "users",
@@ -41,6 +42,129 @@ func TestAddColumn(t *testing.T) {
 						},
 					},
 				},
+				{
+					Name:          "02_add_column",
+					VersionSchema: "add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Column: migrations.Column{
+								Name:     "age",
+								Type:     "integer",
+								Nullable: false,
+								Default:  ptr("0"),
+								Comment:  ptr("the age of the user"),
+							},
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// old and new views of the table should exist
+				ViewMustExist(t, db, schema, "add_table", "users")
+				ViewMustExist(t, db, schema, "add_column", "users")
+
+				// inserting via both the old and the new views works
+				MustInsert(t, db, schema, "add_table", "users", map[string]string{
+					"name": "Alice",
+				})
+				MustInsert(t, db, schema, "add_column", "users", map[string]string{
+					"name": "Bob",
+					"age":  "21",
+				})
+
+				// selecting from both the old and the new views works
+				resOld := MustSelect(t, db, schema, "add_table", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "Alice"},
+					{"id": 2, "name": "Bob"},
+				}, resOld)
+				resNew := MustSelect(t, db, schema, "add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "Alice", "age": 0},
+					{"id": 2, "name": "Bob", "age": 21},
+				}, resNew)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The new column has been dropped from the underlying table
+				columnName := migrations.TemporaryName("age")
+				ColumnMustNotExist(t, db, schema, "users", columnName)
+
+				// The table's column count reflects the drop of the new column
+				TableMustHaveColumnCount(t, db, schema, "users", 2)
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// The new view still exists
+				ViewMustExist(t, db, schema, "add_column", "users")
+
+				// Inserting into the new view still works
+				MustInsert(t, db, schema, "add_column", "users", map[string]string{
+					"name": "Carl",
+					"age":  "31",
+				})
+
+				// Selecting from the new view still works
+				res := MustSelect(t, db, schema, "add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "Alice", "age": 0},
+					{"id": 2, "name": "Bob", "age": 0},
+					{"id": 3, "name": "Carl", "age": 31},
+				}, res)
+			},
+		},
+		{
+			name: "a newly added column does not have to be used as the identity column for a backfill",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_add_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Column: migrations.Column{
+								Name:     "description",
+								Type:     "integer",
+								Nullable: false,
+								Unique:   true,
+							},
+							Up: "'this is a description'",
+						},
+					},
+				},
+			},
+			wantStartErr: nil,
+		},
+		{
+			name: "add column to table with no identity or unique column",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_add_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name:   "name",
+									Type:   "varchar(255)",
+									Unique: true,
+								},
+							},
+						},
+					},
+				},
 				{
 					Name: "02_add_column",
 					Operations: migrations.Operations{
@@ -74,13 +198,13 @@ func TestAddColumn(t *testing.T) {
 				// selecting from both the old and the new views works
 				resOld := MustSelect(t, db, schema, "01_add_table", "users")
 				assert.Equal(t, []map[string]any{
-					{"id": 1, "name": "Alice"},
-					{"id": 2, "name": "Bob"},
+					{"name": "Alice"},
+					{"name": "Bob"},
 				}, resOld)
 				resNew := MustSelect(t, db, schema, "02_add_column", "users")
 				assert.Equal(t, []map[string]any{
-					{"id": 1, "name": "Alice", "age": 0},
-					{"id": 2, "name": "Bob", "age": 21},
+					{"name": "Alice", "age": 0},
+					{"name": "Bob", "age": 21},
 				}, resNew)
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
@@ -89,7 +213,7 @@ func TestAddColumn(t *testing.T) {
 				ColumnMustNotExist(t, db, schema, "users", columnName)
 
 				// The table's column count reflects the drop of the new column
-				TableMustHaveColumnCount(t, db, schema, "users", 2)
+				TableMustHaveColumnCount(t, db, schema, "users", 1)
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
 				// The new view still exists
@@ -104,53 +228,12 @@ func TestAddColumn(t *testing.T) {
 				// Selecting from the new view still works
 				res := MustSelect(t, db, schema, "02_add_column", "users")
 				assert.Equal(t, []map[string]any{
-					{"id": 1, "name": "Alice", "age": 0},
-					{"id": 2, "name": "Bob", "age": 0},
-					{"id": 3, "name": "Carl", "age": 31},
+					{"name": "Alice", "age": 0},
+					{"name": "Bob", "age": 0},
+					{"name": "Carl", "age": 31},
 				}, res)
 			},
 		},
-		{
-			name: "a newly added column can't be used as the identity column for a backfill",
-			migrations: []migrations.Migration{
-				{
-					Name: "01_add_table",
-					Operations: migrations.Operations{
-						&migrations.OpCreateTable{
-							Name: "users",
-							Columns: []migrations.Column{
-								{
-									Name: "name",
-									Type: "varchar(255)",
-								},
-							},
-						},
-					},
-				},
-				{
-					// This column is NOT NULL and UNIQUE and is added to a table without
-					// a PK, so it could in theory be used as the identity column for a
-					// backfill. However, it shouldn't be used as the identity column for
-					// a backfill because it's a newly added column whose temporary
-					// `_pgroll_new_description` column will be full of NULLs for any
-					// existing rows in the table.
-					Name: "02_add_column",
-					Operations: migrations.Operations{
-						&migrations.OpAddColumn{
-							Table: "users",
-							Column: migrations.Column{
-								Name:     "description",
-								Type:     "integer",
-								Nullable: false,
-								Unique:   true,
-							},
-							Up: "'this is a description'",
-						},
-					},
-				},
-			},
-			wantStartErr: backfill.NotPossibleError{Table: "users"},
-		},
 		{
 			name: "add serial columns",
 			migrations: []migrations.Migration{
@@ -776,11 +859,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
@@ -792,11 +875,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 				}, res)
 
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 		},
@@ -858,11 +941,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
@@ -874,11 +957,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 				}, res)
 
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 		},
@@ -940,11 +1023,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
@@ -956,11 +1039,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 				}, res)
 
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 		},
@@ -1024,11 +1107,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
@@ -1040,11 +1123,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 				}, res)
 
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 		},
@@ -1115,11 +1198,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
@@ -1132,11 +1215,11 @@ func TestAddColumnWithUpSql(t *testing.T) {
 				}, res)
 
 				// The trigger function has been dropped.
-				triggerFnName := migrations.TriggerFunctionName("products", "description")
+				triggerFnName := backfill.TriggerFunctionName("products", "description")
 				FunctionMustNotExist(t, db, schema, triggerFnName)
 
 				// The trigger has been dropped.
-				triggerName := migrations.TriggerName("products", "description")
+				triggerName := backfill.TriggerName("products", "description")
 				TriggerMustNotExist(t, db, schema, "products", triggerName)
 			},
 		},
@@ -1920,7 +2003,7 @@ func TestAddColumnValidation(t *testing.T) {
 			wantStartErr: nil,
 		},
 		{
-			name: "table must have a primary key on exactly one column or a unique not null if up is defined",
+			name: "table without a primary key on exactly one column or a unique can be backfilled",
 			migrations: []migrations.Migration{
 				addTableMigrationNoPKNullable,
 				{
@@ -1938,7 +2021,7 @@ func TestAddColumnValidation(t *testing.T) {
 					},
 				},
 			},
-			wantStartErr: backfill.NotPossibleError{Table: "users"},
+			wantStartErr: nil,
 		},
 		{
 			name: "table with a unique not null column can be backfilled",
