@@ -13,9 +13,14 @@ import (
 	"github.com/xataio/pgroll/pkg/schema"
 )
 
-var _ Operation = (*OpAlterColumn)(nil)
+var (
+	_ Operation  = (*OpAlterColumn)(nil)
+	_ Createable = (*OpAlterColumn)(nil)
+)
+
+func (o *OpAlterColumn) Start(ctx context.Context, l Logger, conn db.DB, s *schema.Schema) (*StartResult, error) {
+	l.LogOperationStart(o)
 
-func (o *OpAlterColumn) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
 	table := s.GetTable(o.Table)
 	if table == nil {
 		return nil, TableDoesNotExistError{Name: o.Table}
@@ -29,168 +34,142 @@ func (o *OpAlterColumn) Start(ctx context.Context, conn db.DB, latestSchema stri
 	// Duplicate the column on the underlying table.
 	d := duplicatorForOperations(ops, conn, table, column).
 		WithName(column.Name, TemporaryName(o.Column))
-	if err := d.Duplicate(ctx); err != nil {
+	if err := d.Execute(ctx); err != nil {
 		return nil, fmt.Errorf("failed to duplicate column: %w", err)
 	}
 
-	// Add a trigger to copy values from the old column to the new, rewriting values using the `up` SQL.
-	err := createTrigger(ctx, conn, triggerConfig{
-		Name:           TriggerName(o.Table, o.Column),
-		Direction:      TriggerDirectionUp,
-		Columns:        table.Columns,
-		SchemaName:     s.Name,
-		LatestSchema:   latestSchema,
-		TableName:      table.Name,
-		PhysicalColumn: TemporaryName(o.Column),
-		SQL:            o.upSQLForOperations(ops),
-	})
-	if err != nil {
-		return nil, fmt.Errorf("failed to create up trigger: %w", err)
+	// Copy the columns from table columns, so we can use it later
+	// in the down trigger with the physical name
+	upColumns := make(map[string]*schema.Column)
+	for name, col := range table.Columns {
+		upColumns[name] = col
 	}
 
+	// Add a trigger to copy values from the old column to the new, rewriting values using the `up` SQL.
+	triggers := make([]backfill.OperationTrigger, 0)
+	triggers = append(triggers,
+		backfill.OperationTrigger{
+			Name:           backfill.TriggerName(o.Table, o.Column),
+			Direction:      backfill.TriggerDirectionUp,
+			TableName:      table.Name,
+			Columns:        upColumns,
+			PhysicalColumn: TemporaryName(o.Column),
+			SQL:            o.upSQLForOperations(ops),
+		},
+	)
+
 	// Add the new column to the internal schema representation. This is done
 	// here, before creation of the down trigger, so that the trigger can declare
 	// a variable for the new column. Save the old column name for use as the
 	// physical column name. in the down trigger first.
 	oldPhysicalColumn := column.Name
+	columnType := column.Type
+	if o.Type != nil {
+		columnType = *o.Type
+	}
 	table.AddColumn(o.Column, &schema.Column{
 		Name: TemporaryName(o.Column),
+		Type: columnType,
 	})
 
 	// Add a trigger to copy values from the new column to the old.
-	err = createTrigger(ctx, conn, triggerConfig{
-		Name:           TriggerName(o.Table, TemporaryName(o.Column)),
-		Direction:      TriggerDirectionDown,
-		Columns:        table.Columns,
-		LatestSchema:   latestSchema,
-		SchemaName:     s.Name,
-		TableName:      table.Name,
-		PhysicalColumn: oldPhysicalColumn,
-		SQL:            o.downSQLForOperations(ops),
-	})
-	if err != nil {
-		return nil, fmt.Errorf("failed to create down trigger: %w", err)
-	}
+	triggers = append(triggers,
+		backfill.OperationTrigger{
+			Name:           backfill.TriggerName(o.Table, TemporaryName(o.Column)),
+			Direction:      backfill.TriggerDirectionDown,
+			TableName:      table.Name,
+			Columns:        table.Columns,
+			PhysicalColumn: oldPhysicalColumn,
+			SQL:            o.downSQLForOperations(ops),
+		},
+	)
+	task := backfill.NewTask(table, triggers...)
 
+	var dbActions []DBAction
 	// perform any operation specific start steps
 	for _, op := range ops {
-		if _, err := op.Start(ctx, conn, latestSchema, s); err != nil {
+		startOp, err := op.Start(ctx, l, conn, s)
+		if err != nil {
 			return nil, err
 		}
+		task.AddTriggers(startOp.BackfillTask)
+		dbActions = append(dbActions, startOp.Actions...)
 	}
 
-	return table, nil
+	return &StartResult{Actions: dbActions, BackfillTask: task}, nil
 }
 
-func (o *OpAlterColumn) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
+func (o *OpAlterColumn) Complete(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationComplete(o)
+
 	ops := o.subOperations()
 
+	dbActions := make([]DBAction, 0)
 	// Perform any operation specific completion steps
 	for _, op := range ops {
-		if err := op.Complete(ctx, conn, s); err != nil {
-			return err
+		actions, err := op.Complete(l, conn, s)
+		if err != nil {
+			return nil, err
 		}
+		dbActions = append(dbActions, actions...)
 	}
 
-	if err := alterSequenceOwnerToDuplicatedColumn(ctx, conn, o.Table, o.Column); err != nil {
-		return err
+	table := s.GetTable(o.Table)
+	if table == nil {
+		return nil, TableDoesNotExistError{Name: o.Table}
 	}
-
-	// Drop the old column
-	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
-		pq.QuoteIdentifier(o.Table),
-		pq.QuoteIdentifier(o.Column)))
-	if err != nil {
-		return err
-	}
-
-	// Remove the up function and trigger
-	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, o.Column))))
-	if err != nil {
-		return err
-	}
-
-	// Remove the down function and trigger
-	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(o.Column)))))
-	if err != nil {
-		return err
-	}
-
-	// Remove the needs backfill column
-	_, err = conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
-		pq.QuoteIdentifier(o.Table),
-		pq.QuoteIdentifier(CNeedsBackfillColumn)))
-	if err != nil {
-		return err
+	column := table.GetColumn(o.Column)
+	if column == nil {
+		return nil, ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
 	}
+	table.Name = o.Table
 
 	// Rename the new column to the old column name
-	table := s.GetTable(o.Table)
-	if table == nil {
-		return TableDoesNotExistError{Name: o.Table}
-	}
-	column := table.GetColumn(o.Column)
-	if column == nil {
-		return ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
-	}
-	if err := RenameDuplicatedColumn(ctx, conn, table, column); err != nil {
-		return err
-	}
-
-	return nil
+	return append(dbActions, []DBAction{
+		NewAlterSequenceOwnerAction(conn, table.Name, column.Name, TemporaryName(column.Name)),
+		NewDropColumnAction(conn, table.Name, o.Column),
+		NewDropFunctionAction(conn,
+			backfill.TriggerFunctionName(o.Table, o.Column),
+			backfill.TriggerFunctionName(o.Table, TemporaryName(o.Column)),
+		),
+		NewDropColumnAction(conn, o.Table, backfill.CNeedsBackfillColumn),
+		NewRenameDuplicatedColumnAction(conn, table, o.Column),
+	}...), nil
 }
 
-func (o *OpAlterColumn) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
+func (o *OpAlterColumn) Rollback(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationRollback(o)
+
 	table := s.GetTable(o.Table)
 	if table == nil {
-		return TableDoesNotExistError{Name: o.Table}
+		return nil, TableDoesNotExistError{Name: o.Table}
 	}
 	column := table.GetColumn(o.Column)
 	if column == nil {
-		return ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
+		return nil, ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
 	}
 
 	// Perform any operation specific rollback steps
+	dbActions := make([]DBAction, 0)
 	ops := o.subOperations()
 	for _, ops := range ops {
-		if err := ops.Rollback(ctx, conn, nil); err != nil {
-			return err
+		actions, err := ops.Rollback(l, conn, nil)
+		if err != nil {
+			return nil, err
 		}
+		dbActions = append(dbActions, actions...)
 	}
 
-	// Drop the new column
-	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s DROP COLUMN IF EXISTS %s",
-		pq.QuoteIdentifier(table.Name),
-		pq.QuoteIdentifier(column.Name),
-	))
-	if err != nil {
-		return err
-	}
+	dbActions = append(dbActions,
+		NewDropColumnAction(conn, table.Name, column.Name),
+		NewDropFunctionAction(conn,
+			backfill.TriggerFunctionName(o.Table, o.Column),
+			backfill.TriggerFunctionName(o.Table, TemporaryName(o.Column)),
+		),
+		NewDropColumnAction(conn, table.Name, backfill.CNeedsBackfillColumn),
+	)
 
-	// Remove the up function and trigger
-	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, o.Column)),
-	))
-	if err != nil {
-		return err
-	}
-
-	// Remove the down function and trigger
-	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(o.Column))),
-	))
-	if err != nil {
-		return err
-	}
-
-	// Remove the needs backfill column
-	_, err = conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
-		pq.QuoteIdentifier(table.Name),
-		pq.QuoteIdentifier(CNeedsBackfillColumn)))
-
-	return err
+	return dbActions, nil
 }
 
 func (o *OpAlterColumn) Validate(ctx context.Context, s *schema.Schema) error {
@@ -203,12 +182,6 @@ func (o *OpAlterColumn) Validate(ctx context.Context, s *schema.Schema) error {
 		return ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
 	}
 
-	// If the operation requires backfills (ie it isn't a rename-only operation),
-	// ensure that the column meets the requirements for backfilling.
-	if err := backfill.IsPossible(table); err != nil {
-		return err
-	}
-
 	ops := o.subOperations()
 
 	// Ensure that at least one sub-operation or rename is present
@@ -315,7 +288,7 @@ func (o *OpAlterColumn) subOperations() []Operation {
 }
 
 // duplicatorForOperations returns a Duplicator for the given operations
-func duplicatorForOperations(ops []Operation, conn db.DB, table *schema.Table, column *schema.Column) *Duplicator {
+func duplicatorForOperations(ops []Operation, conn db.DB, table *schema.Table, column *schema.Column) *duplicator {
 	d := NewColumnDuplicator(conn, table, column)
 
 	for _, op := range ops {
