@@ -21,7 +21,8 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 			name: "can alter a column: set not null, change type, change comment, and add check constraint",
 			migrations: []migrations.Migration{
 				{
-					Name: "01_create_table",
+					Name:          "01_create_table",
+					VersionSchema: "create_table",
 					Operations: migrations.Operations{
 						&migrations.OpCreateTable{
 							Name: "events",
@@ -41,7 +42,8 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 					},
 				},
 				{
-					Name: "02_alter_column",
+					Name:          "02_alter_column",
+					VersionSchema: "alter_column",
 					Operations: migrations.Operations{
 						&migrations.OpAlterColumn{
 							Table:    "events",
@@ -61,45 +63,45 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 			},
 			afterStart: func(t *testing.T, db *sql.DB, schema string) {
 				// Inserting a NULL into the new column should fail
-				MustNotInsert(t, db, schema, "02_alter_column", "events", map[string]string{
+				MustNotInsert(t, db, schema, "alter_column", "events", map[string]string{
 					"id": "1",
 				}, testutils.CheckViolationErrorCode)
 
 				// Inserting a non-NULL value into the new column should succeed
-				MustInsert(t, db, schema, "02_alter_column", "events", map[string]string{
+				MustInsert(t, db, schema, "alter_column", "events", map[string]string{
 					"id":   "2",
 					"name": "apples",
 				})
 
 				// The value inserted into the new column has been backfilled into the
 				// old column.
-				rows := MustSelect(t, db, schema, "01_create_table", "events")
+				rows := MustSelect(t, db, schema, "create_table", "events")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "name": "apples"},
 				}, rows)
 
 				// Inserting a NULL value into the old column should succeed
-				MustInsert(t, db, schema, "01_create_table", "events", map[string]string{
+				MustInsert(t, db, schema, "create_table", "events", map[string]string{
 					"id": "3",
 				})
 
 				// The NULL value inserted into the old column has been written into
 				// the new column using the `up` SQL.
-				rows = MustSelect(t, db, schema, "02_alter_column", "events")
+				rows = MustSelect(t, db, schema, "alter_column", "events")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "name": "apples"},
 					{"id": 3, "name": "placeholder"},
 				}, rows)
 
 				// Inserting a non-NULL value into the old column should succeed
-				MustInsert(t, db, schema, "01_create_table", "events", map[string]string{
+				MustInsert(t, db, schema, "create_table", "events", map[string]string{
 					"id":   "4",
 					"name": "bananas",
 				})
 
 				// The non-NULL value inserted into the old column has been copied
 				// unchanged into the new column.
-				rows = MustSelect(t, db, schema, "02_alter_column", "events")
+				rows = MustSelect(t, db, schema, "alter_column", "events")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "name": "apples"},
 					{"id": 3, "name": "placeholder"},
@@ -108,21 +110,21 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 
 				// Inserting a row into the new column that violates the
 				// check constraint should fail
-				MustNotInsert(t, db, schema, "02_alter_column", "events", map[string]string{
+				MustNotInsert(t, db, schema, "alter_column", "events", map[string]string{
 					"id":   "5",
 					"name": "x",
 				}, testutils.CheckViolationErrorCode)
 
 				// Inserting a row into the old column that violates the
 				// check constraint should succeed.
-				MustInsert(t, db, schema, "01_create_table", "events", map[string]string{
+				MustInsert(t, db, schema, "create_table", "events", map[string]string{
 					"id":   "5",
 					"name": "x",
 				})
 
 				// The value that didn't meet the check constraint has been rewritten
 				// into the new column using the `up` SQL.
-				rows = MustSelect(t, db, schema, "02_alter_column", "events")
+				rows = MustSelect(t, db, schema, "alter_column", "events")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "name": "apples"},
 					{"id": 3, "name": "placeholder"},
@@ -142,13 +144,13 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
 				// Inserting a NULL into the new column should fail
-				MustNotInsert(t, db, schema, "02_alter_column", "events", map[string]string{
+				MustNotInsert(t, db, schema, "alter_column", "events", map[string]string{
 					"id": "6",
 				}, testutils.NotNullViolationErrorCode)
 
 				// Inserting a row into the new column that violates the
 				// check constraint should fail
-				MustNotInsert(t, db, schema, "02_alter_column", "events", map[string]string{
+				MustNotInsert(t, db, schema, "alter_column", "events", map[string]string{
 					"id":   "6",
 					"name": "x",
 				}, testutils.CheckViolationErrorCode)
@@ -160,7 +162,7 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 				ColumnMustHaveComment(t, db, schema, "events", "name", "the name of the event")
 
 				// The table contains the expected rows
-				rows := MustSelect(t, db, schema, "02_alter_column", "events")
+				rows := MustSelect(t, db, schema, "alter_column", "events")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "name": "apples"},
 					{"id": 3, "name": "placeholder"},
@@ -211,6 +213,11 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 					"id": "1",
 				})
 
+				// Updating name to default value should succeed
+				MustUpdate(t, db, schema, "02_alter_column", "events", "id", "1", map[string]string{
+					"name": "DEFAULT",
+				})
+
 				// Inserting a NULL explicitly into the new schema should fail
 				MustNotInsert(t, db, schema, "02_alter_column", "events", map[string]string{
 					"id":   "100",
@@ -265,6 +272,11 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 					"id": "4",
 				})
 
+				// Updating name to default value should succeed
+				MustUpdate(t, db, schema, "02_alter_column", "events", "id", "4", map[string]string{
+					"name": "DEFAULT",
+				})
+
 				// Inserting a NULL explicitly into the new schema should fail
 				MustNotInsert(t, db, schema, "02_alter_column", "events", map[string]string{
 					"id":   "100",
@@ -493,6 +505,279 @@ func TestAlterColumnMultipleSubOperations(t *testing.T) {
 				}, rows)
 			},
 		},
+		{
+			name: "can alter a column multiple times: add check and not null",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "people",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_alter_column",
+					Operations: migrations.Operations{
+						&migrations.OpAlterColumn{
+							Table:  "people",
+							Column: "name",
+							Up:     "INITCAP(name)",
+							Down:   "name",
+							Check: &migrations.CheckConstraint{
+								Name:       "name_capitalized",
+								Constraint: "name ~ '^[A-Z].*'",
+							},
+						},
+						&migrations.OpAlterColumn{
+							Table:    "people",
+							Column:   "name",
+							Up:       "SELECT CASE WHEN name IS NULL THEN 'Jane' ELSE name END",
+							Down:     "name",
+							Nullable: ptr(false),
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the `people` table with a lowercase `name` value
+				MustInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"name": "alice",
+				})
+				// Can insert a row into the `people` table with a NULL `name` value
+				MustInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"id": "2",
+				})
+				// Cannot insert a row into the `people` table with an lowercase `name` value
+				MustNotInsert(t, db, schema, "02_alter_column", "people", map[string]string{
+					"name": "bob",
+				}, testutils.CheckViolationErrorCode)
+				// Can insert a row into the `people` table with a missing `name` value
+				MustNotInsert(t, db, schema, "02_alter_column", "people", map[string]string{
+					"id": "4",
+				}, testutils.CheckViolationErrorCode)
+				// Can insert a row into the `people` table with a capitalized `name` value
+				MustInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"id":   "4",
+					"name": "Carl",
+				})
+
+				// The version of the `people` table in the new schema has the expected rows.
+				rows := MustSelect(t, db, schema, "02_alter_column", "people")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "Alice"},
+					{"id": 2, "name": "Jane"},
+					{"id": 4, "name": "Carl"},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the `people` table with a lowercase `name` value
+				MustInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"name": "alice",
+				})
+				// Can insert a row into the `people` table with a NULL `name` value
+				MustInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"id": "100",
+				})
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting a row into the `people` table with a NULL `manages` value fails
+				MustNotInsert(t, db, schema, "02_alter_column", "people", map[string]string{
+					"name": "carl",
+				}, testutils.CheckViolationErrorCode)
+
+				// The `people` table has the expected rows.
+				rows := MustSelect(t, db, schema, "02_alter_column", "people")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "Alice"},
+					{"id": 2, "name": "Jane"},
+					{"id": 4, "name": "Carl"},
+					{"id": 3, "name": "Alice"},
+					{"id": 100, "name": "Jane"},
+				}, rows)
+			},
+		},
+	})
+}
+
+func TestAlterPrimaryKeyColumns(t *testing.T) {
+	t.Parallel()
+
+	ExecuteTests(t, TestCases{
+		{
+			name: "alter a single primary key column",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "people",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_alter_column",
+					Operations: migrations.Operations{
+						&migrations.OpAlterColumn{
+							Table:  "people",
+							Column: "id",
+							Type:   ptr("bigint"),
+							Up:     "CAST(id AS bigint)",
+							Down:   "SELECT CASE WHEN id < 2147483647 THEN CAST(id AS int) ELSE 0 END",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				PrimaryKeyConstraintMustExist(t, db, schema, "people", "people_pkey")
+				ColumnMustBePK(t, db, schema, "people", "id")
+
+				bigint := "31474836471" // A value larger than int can hold
+				integer := "100"
+
+				// Inserting a row with integer id into the old schema should succeed
+				MustInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"id":   integer,
+					"name": "alice",
+				})
+				// Inserting a row with integer bigint id into the old schema should fail
+				MustNotInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"id":   bigint,
+					"name": "alice",
+				}, testutils.NumericValueOutOfRangeErrorCode)
+
+				// Inserting a row with bigint id into the new schema should succeed
+				MustInsert(t, db, schema, "02_alter_column", "people", map[string]string{
+					"id":   bigint,
+					"name": "alice",
+				})
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				PrimaryKeyConstraintMustExist(t, db, schema, "people", "people_pkey")
+				ColumnMustBePK(t, db, schema, "people", "id")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				PrimaryKeyConstraintMustExist(t, db, schema, "people", "people_pkey")
+				ColumnMustBePK(t, db, schema, "people", "id")
+
+				// Inserting a row with integer bigint into the new schema should succeed
+				MustInsert(t, db, schema, "02_alter_column", "people", map[string]string{
+					"id":   "31474836472",
+					"name": "bob",
+				})
+			},
+		},
+		{
+			name: "alter a composite primary key: one column changes type, the other is unchanged",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "people",
+							Columns: []migrations.Column{
+								{
+									Name: "id1",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "id2",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_alter_column",
+					Operations: migrations.Operations{
+						&migrations.OpAlterColumn{
+							Table:  "people",
+							Column: "id1",
+							Type:   ptr("bigint"),
+							Up:     "CAST(id1 AS bigint)",
+							Down:   "SELECT CASE WHEN id1 < 2147483647 THEN CAST(id1 AS int) ELSE 0 END",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				PrimaryKeyConstraintMustExist(t, db, schema, "people", "people_pkey")
+				ColumnMustBePK(t, db, schema, "people", "id1")
+				ColumnMustBePK(t, db, schema, "people", "id2")
+
+				bigint := "31474836471" // A value larger than int can hold
+				integer := "100"
+
+				// Inserting a row with integer id into the old schema should succeed
+				MustInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"id1":  integer,
+					"id2":  integer,
+					"name": "alice",
+				})
+				// Inserting a row with integer bigint id into the old schema should fail
+				MustNotInsert(t, db, schema, "01_create_table", "people", map[string]string{
+					"id1":  bigint,
+					"id2":  integer,
+					"name": "alice",
+				}, testutils.NumericValueOutOfRangeErrorCode)
+
+				// Inserting a row with bigint id into the new schema should succeed
+				MustInsert(t, db, schema, "02_alter_column", "people", map[string]string{
+					"id1":  bigint,
+					"id2":  integer,
+					"name": "alice",
+				})
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				PrimaryKeyConstraintMustExist(t, db, schema, "people", "people_pkey")
+				ColumnMustBePK(t, db, schema, "people", "id1")
+				ColumnMustBePK(t, db, schema, "people", "id2")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				PrimaryKeyConstraintMustExist(t, db, schema, "people", "people_pkey")
+				ColumnMustBePK(t, db, schema, "people", "id1")
+				ColumnMustBePK(t, db, schema, "people", "id2")
+
+				// Inserting a row with integer bigint into the new schema should succeed
+				MustInsert(t, db, schema, "02_alter_column", "people", map[string]string{
+					"id1":  "31474836472",
+					"id2":  "72",
+					"name": "bob",
+				})
+			},
+		},
 	})
 }
 
