@@ -5,6 +5,7 @@ package migrations
 import (
 	"context"
 
+	"github.com/xataio/pgroll/pkg/backfill"
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -19,21 +20,27 @@ type OpChangeType struct {
 
 var _ Operation = (*OpChangeType)(nil)
 
-func (o *OpChangeType) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
+func (o *OpChangeType) Start(ctx context.Context, l Logger, conn db.DB, s *schema.Schema) (*StartResult, error) {
+	l.LogOperationStart(o)
+
 	table := s.GetTable(o.Table)
 	if table == nil {
 		return nil, TableDoesNotExistError{Name: o.Table}
 	}
 
-	return table, nil
+	return &StartResult{BackfillTask: backfill.NewTask(table)}, nil
 }
 
-func (o *OpChangeType) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
-	return nil
+func (o *OpChangeType) Complete(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationComplete(o)
+
+	return nil, nil
 }
 
-func (o *OpChangeType) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
-	return nil
+func (o *OpChangeType) Rollback(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationRollback(o)
+
+	return nil, nil
 }
 
 func (o *OpChangeType) Validate(ctx context.Context, s *schema.Schema) error {
