@@ -7,6 +7,13 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
+	"io/fs"
+	"maps"
+	"path/filepath"
+	"slices"
+	"strings"
+
+	"sigs.k8s.io/yaml"
 )
 
 type OpName string
@@ -29,6 +36,25 @@ const (
 	OpCreateConstraintName          OpName = "create_constraint"
 )
 
+// AllNonDeprecatedOperations contains the list of operations
+// that are not deprecated. These operations must implement
+// migrations.Createable interface.
+var AllNonDeprecatedOperations = []string{
+	string(OpNameCreateTable),
+	string(OpNameRenameTable),
+	string(OpNameRenameColumn),
+	string(OpNameDropTable),
+	string(OpNameAddColumn),
+	string(OpNameDropColumn),
+	string(OpNameAlterColumn),
+	string(OpNameCreateIndex),
+	string(OpNameDropIndex),
+	string(OpNameRenameConstraint),
+	string(OpNameDropMultiColumnConstraint),
+	string(OpRawSQLName),
+	string(OpCreateConstraintName),
+}
+
 const (
 	temporaryPrefix = "_pgroll_new_"
 	deletedPrefix   = "_pgroll_del_"
@@ -44,22 +70,82 @@ func DeletionName(name string) string {
 	return deletedPrefix + name
 }
 
-// ReadMigration reads a migration from an io.Reader, like a file.
-func ReadMigration(r io.Reader) (*Migration, error) {
-	byteValue, err := io.ReadAll(r)
+// CollectFilesFromDir returns a list of migration files in a directory.
+// The files are ordered based on the filename without the extension name.
+func CollectFilesFromDir(dir fs.FS) ([]string, error) {
+	supportedExtensionsGlob := []string{"*.json", "*.yml", "*.yaml"}
+	var migrationFiles []string
+	for _, glob := range supportedExtensionsGlob {
+		files, err := fs.Glob(dir, glob)
+		if err != nil {
+			return nil, fmt.Errorf("reading directory: %w", err)
+		}
+		migrationFiles = append(migrationFiles, files...)
+	}
+
+	// Order slice based on filename without extension
+	slices.SortFunc(migrationFiles, func(f1, f2 string) int {
+		return strings.Compare(filepath.Base(f1), filepath.Base(f2))
+	})
+
+	return migrationFiles, nil
+}
+
+// ReadMigration opens the migration file and reads the migration as a
+// RawMigration.
+func ReadRawMigration(dir fs.FS, filename string) (*RawMigration, error) {
+	file, err := dir.Open(filename)
+	if err != nil {
+		return nil, fmt.Errorf("opening migration file: %w", err)
+	}
+	defer file.Close()
+
+	byteValue, err := io.ReadAll(file)
 	if err != nil {
 		return nil, err
 	}
 
-	dec := json.NewDecoder(bytes.NewReader(byteValue))
-	dec.DisallowUnknownFields()
+	mig := RawMigration{}
+	switch filepath.Ext(filename) {
+	case ".json":
+		dec := json.NewDecoder(bytes.NewReader(byteValue))
+		dec.DisallowUnknownFields()
+		err = dec.Decode(&mig)
+	case ".yaml", ".yml":
+		err = yaml.UnmarshalStrict(byteValue, &mig)
+	}
+	if err != nil {
+		return nil, fmt.Errorf("reading migration file: %w", err)
+	}
 
-	mig := Migration{}
-	if err = dec.Decode(&mig); err != nil {
+	// Extract base filename without extension as the migration name
+	mig.Name = strings.TrimSuffix(filepath.Base(filename), filepath.Ext(filename))
+
+	return &mig, nil
+}
+
+// ParseMigration converts a RawMigration to a fully parsed Migration
+func ParseMigration(raw *RawMigration) (*Migration, error) {
+	var ops Operations
+	if err := json.Unmarshal(raw.Operations, &ops); err != nil {
+		return nil, fmt.Errorf("parsing operations: %w", err)
+	}
+
+	return &Migration{
+		Name:          raw.Name,
+		VersionSchema: raw.VersionSchema,
+		Operations:    ops,
+	}, nil
+}
+
+// ReadMigration reads and parses a migration file
+func ReadMigration(dir fs.FS, filename string) (*Migration, error) {
+	raw, err := ReadRawMigration(dir, filename)
+	if err != nil {
 		return nil, err
 	}
 
-	return &mig, nil
+	return ParseMigration(raw)
 }
 
 // UnmarshalJSON deserializes the list of operations from a JSON array.
@@ -79,62 +165,17 @@ func (v *Operations) UnmarshalJSON(data []byte) error {
 		var opName OpName
 		var logBody json.RawMessage
 		if len(opObj) != 1 {
-			return fmt.Errorf("invalid migration: %v", opObj)
+			return fmt.Errorf("multiple keys in operation object at index %d: %v",
+				i, strings.Join(slices.Collect(maps.Keys(opObj)), ", "))
 		}
 		for k, v := range opObj {
 			opName = OpName(k)
 			logBody = v
 		}
 
-		var item Operation
-		switch opName {
-		case OpNameCreateTable:
-			item = &OpCreateTable{}
-
-		case OpNameRenameTable:
-			item = &OpRenameTable{}
-
-		case OpNameDropTable:
-			item = &OpDropTable{}
-
-		case OpNameAddColumn:
-			item = &OpAddColumn{}
-
-		case OpNameRenameColumn:
-			item = &OpRenameColumn{}
-
-		case OpNameDropColumn:
-			item = &OpDropColumn{}
-
-		case OpNameRenameConstraint:
-			item = &OpRenameConstraint{}
-
-		case OpNameDropConstraint:
-			item = &OpDropConstraint{}
-
-		case OpNameSetReplicaIdentity:
-			item = &OpSetReplicaIdentity{}
-
-		case OpNameAlterColumn:
-			item = &OpAlterColumn{}
-
-		case OpNameCreateIndex:
-			item = &OpCreateIndex{}
-
-		case OpNameDropIndex:
-			item = &OpDropIndex{}
-
-		case OpRawSQLName:
-			item = &OpRawSQL{}
-
-		case OpCreateConstraintName:
-			item = &OpCreateConstraint{}
-
-		case OpNameDropMultiColumnConstraint:
-			item = &OpDropMultiColumnConstraint{}
-
-		default:
-			return fmt.Errorf("unknown migration type: %v", opName)
+		item, err := OperationFromName(opName)
+		if err != nil {
+			return err
 		}
 
 		dec := json.NewDecoder(bytes.NewReader(logBody))
@@ -229,3 +270,54 @@ func OperationName(op Operation) OpName {
 
 	panic(fmt.Errorf("unknown operation for %T", op))
 }
+
+func OperationFromName(name OpName) (Operation, error) {
+	switch name {
+	case OpNameCreateTable:
+		return &OpCreateTable{}, nil
+
+	case OpNameRenameTable:
+		return &OpRenameTable{}, nil
+
+	case OpNameDropTable:
+		return &OpDropTable{}, nil
+
+	case OpNameAddColumn:
+		return &OpAddColumn{}, nil
+
+	case OpNameRenameColumn:
+		return &OpRenameColumn{}, nil
+
+	case OpNameDropColumn:
+		return &OpDropColumn{}, nil
+
+	case OpNameRenameConstraint:
+		return &OpRenameConstraint{}, nil
+
+	case OpNameDropConstraint:
+		return &OpDropConstraint{}, nil
+
+	case OpNameSetReplicaIdentity:
+		return &OpSetReplicaIdentity{}, nil
+
+	case OpNameAlterColumn:
+		return &OpAlterColumn{}, nil
+
+	case OpNameCreateIndex:
+		return &OpCreateIndex{}, nil
+
+	case OpNameDropIndex:
+		return &OpDropIndex{}, nil
+
+	case OpRawSQLName:
+		return &OpRawSQL{}, nil
+
+	case OpCreateConstraintName:
+		return &OpCreateConstraint{}, nil
+
+	case OpNameDropMultiColumnConstraint:
+		return &OpDropMultiColumnConstraint{}, nil
+
+	}
+	return nil, fmt.Errorf("unknown migration type: %v", name)
+}
