@@ -21,7 +21,8 @@ func TestCreateConstraint(t *testing.T) {
 			name: "create unique constraint on single column",
 			migrations: []migrations.Migration{
 				{
-					Name: "01_add_table",
+					Name:          "01_add_table",
+					VersionSchema: "add_table",
 					Operations: migrations.Operations{
 						&migrations.OpCreateTable{
 							Name: "users",
@@ -41,7 +42,8 @@ func TestCreateConstraint(t *testing.T) {
 					},
 				},
 				{
-					Name: "02_create_constraint",
+					Name:          "02_create_constraint",
+					VersionSchema: "create_constraint",
 					Operations: migrations.Operations{
 						&migrations.OpCreateConstraint{
 							Name:    "unique_name",
@@ -63,18 +65,18 @@ func TestCreateConstraint(t *testing.T) {
 				IndexMustExist(t, db, schema, "users", "unique_name")
 
 				// Inserting values into the old schema that violate uniqueness should succeed.
-				MustInsert(t, db, schema, "01_add_table", "users", map[string]string{
+				MustInsert(t, db, schema, "add_table", "users", map[string]string{
 					"name": "alice",
 				})
-				MustInsert(t, db, schema, "01_add_table", "users", map[string]string{
+				MustInsert(t, db, schema, "add_table", "users", map[string]string{
 					"name": "alice",
 				})
 
 				// Inserting values into the new schema that violate uniqueness should fail.
-				MustInsert(t, db, schema, "02_create_constraint", "users", map[string]string{
+				MustInsert(t, db, schema, "create_constraint", "users", map[string]string{
 					"name": "bob",
 				})
-				MustNotInsert(t, db, schema, "02_create_constraint", "users", map[string]string{
+				MustNotInsert(t, db, schema, "create_constraint", "users", map[string]string{
 					"name": "bob",
 				}, testutils.UniqueViolationErrorCode)
 			},
@@ -90,10 +92,10 @@ func TestCreateConstraint(t *testing.T) {
 				TableMustBeCleanedUp(t, db, schema, "users", "name")
 
 				// Inserting values into the new schema that violate uniqueness should fail.
-				MustInsert(t, db, schema, "02_create_constraint", "users", map[string]string{
+				MustInsert(t, db, schema, "create_constraint", "users", map[string]string{
 					"name": "carol",
 				})
-				MustNotInsert(t, db, schema, "02_create_constraint", "users", map[string]string{
+				MustNotInsert(t, db, schema, "create_constraint", "users", map[string]string{
 					"name": "carol",
 				}, testutils.UniqueViolationErrorCode)
 			},
@@ -738,6 +740,92 @@ func TestCreateConstraint(t *testing.T) {
 				}, testutils.UniqueViolationErrorCode)
 			},
 		},
+		{
+			name: "create primary key constraint on multiple columns",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_add_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "id1",
+									Type: "uuid",
+								},
+								{
+									Name: "id2",
+									Type: "uuid",
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: false,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_create_constraint",
+					Operations: migrations.Operations{
+						&migrations.OpCreateConstraint{
+							Name:    "id_pkey",
+							Table:   "users",
+							Type:    "primary_key",
+							Columns: []string{"id1", "id2"},
+							Up: map[string]string{
+								"id1": "gen_random_uuid()",
+								"id2": "gen_random_uuid()",
+							},
+							Down: map[string]string{
+								"id1": "id1",
+								"id2": "id2",
+							},
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// The index has been created on the underlying table.
+				IndexMustExist(t, db, schema, "users", "id_pkey")
+
+				// Inserting values into the old schema that violate uniqueness should succeed.
+				MustInsert(t, db, schema, "01_add_table", "users", map[string]string{
+					"id1":  "00000000-0000-0000-0000-000000000001",
+					"id2":  "00000000-0000-0000-0000-000000000002",
+					"name": "alice",
+				})
+				MustInsert(t, db, schema, "01_add_table", "users", map[string]string{
+					"id1":  "00000000-0000-0000-0000-000000000001",
+					"id2":  "00000000-0000-0000-0000-000000000002",
+					"name": "bob",
+				})
+
+				MustInsert(t, db, schema, "02_create_constraint", "users", map[string]string{
+					"id1":  "00000000-0000-0000-0000-000000000003",
+					"id2":  "00000000-0000-0000-0000-000000000004",
+					"name": "alice",
+				})
+				// Inserting values into the new schema that violate uniqueness should fail.
+				MustNotInsert(t, db, schema, "02_create_constraint", "users", map[string]string{
+					"id1":  "00000000-0000-0000-0000-000000000003",
+					"id2":  "00000000-0000-0000-0000-000000000004",
+					"name": "bob",
+				}, testutils.UniqueViolationErrorCode)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The index has been dropped from the the underlying table.
+				IndexMustNotExist(t, db, schema, "users", "id_pkey")
+
+				// Functions, triggers and temporary columns are dropped.
+				TableMustBeCleanedUp(t, db, schema, "users", "id1")
+				TableMustBeCleanedUp(t, db, schema, "users", "id2")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				PrimaryKeyConstraintMustExist(t, db, schema, "users", "id_pkey")
+			},
+		},
 	})
 }
 
@@ -1225,6 +1313,105 @@ func TestCreateConstraintInMultiOperationMigrations(t *testing.T) {
 				TableMustBeCleanedUp(t, db, schema, "products", "name")
 			},
 		},
+		{
+			name: "create table with comment and default on column, add unique constraint",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name:    "name",
+									Type:    "text",
+									Default: ptr("'Pixel'"),
+									Comment: ptr("my important comment"),
+								},
+							},
+						},
+						&migrations.OpCreateConstraint{
+							Table:   "items",
+							Type:    migrations.OpCreateConstraintTypeUnique,
+							Name:    "unique_item_name",
+							Columns: []string{"name"},
+							Up: map[string]string{
+								"name": "name",
+							},
+							Down: map[string]string{
+								"name": "name",
+							},
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Column must have comment set
+				ColumnMustHaveComment(t, db, schema, "items", "name", "my important comment")
+				// Column must have default value
+				ColumnMustHaveDefault(t, db, schema, "items", "name", "'Pixel'::text")
+				// Can insert a row into the new schema
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"id":   "1",
+					"name": "apple",
+				})
+
+				// Can insert a row into the new schema that meets the constraint
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"id":   "2",
+					"name": "banana",
+				})
+
+				// Can't insert a row into the new schema that violates the constraint
+				MustNotInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"id":   "3",
+					"name": "apple",
+				}, testutils.UniqueViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "01_create_table", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apple"},
+					{"id": 2, "name": "banana"},
+				}, rows)
+
+				// The old view has the expected rows
+				rows = MustSelect(t, db, schema, "01_create_table", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apple"},
+					{"id": 2, "name": "banana"},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Column comment must be preserved
+				ColumnMustHaveComment(t, db, schema, "items", "name", "my important comment")
+				// Column must have default value
+				ColumnMustHaveDefault(t, db, schema, "items", "name", "'Pixel'::text")
+				// Can insert a row into the new schema that meets the constraint
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"id":   "3",
+					"name": "carrot",
+				})
+
+				// Can't insert a row into the new schema that violates the constraint
+				MustNotInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"id":   "4",
+					"name": "carrot",
+				}, testutils.UniqueViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "01_create_table", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 3, "name": "carrot"},
+				}, rows)
+			},
+		},
 	})
 }
 
