@@ -8,15 +8,18 @@ import (
 	"slices"
 	"strings"
 
-	"github.com/lib/pq"
-
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
 
-var _ Operation = (*OpCreateTable)(nil)
+var (
+	_ Operation  = (*OpCreateTable)(nil)
+	_ Createable = (*OpCreateTable)(nil)
+)
+
+func (o *OpCreateTable) Start(ctx context.Context, l Logger, conn db.DB, s *schema.Schema) (*StartResult, error) {
+	l.LogOperationStart(o)
 
-func (o *OpCreateTable) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
 	// Generate SQL for the columns in the table
 	columnsSQL, err := columnsToSQL(o.Columns)
 	if err != nil {
@@ -28,46 +31,38 @@ func (o *OpCreateTable) Start(ctx context.Context, conn db.DB, latestSchema stri
 		return nil, fmt.Errorf("failed to create constraints SQL: %w", err)
 	}
 
-	// Create the table
-	_, err = conn.ExecContext(ctx, fmt.Sprintf("CREATE TABLE %s (%s %s)",
-		pq.QuoteIdentifier(o.Name),
-		columnsSQL,
-		constraintsSQL))
-	if err != nil {
-		return nil, err
-	}
+	dbActions := make([]DBAction, 0)
+	dbActions = append(dbActions, NewCreateTableAction(conn, o.Name, columnsSQL, constraintsSQL))
 
 	// Add comments to any columns that have them
 	for _, col := range o.Columns {
 		if col.Comment != nil {
-			if err := addCommentToColumn(ctx, conn, o.Name, col.Name, col.Comment); err != nil {
-				return nil, fmt.Errorf("failed to add comment to column: %w", err)
-			}
+			dbActions = append(dbActions, NewCommentColumnAction(conn, o.Name, col.Name, col.Comment))
 		}
 	}
 
 	// Add comment to the table itself
 	if o.Comment != nil {
-		if err := addCommentToTable(ctx, conn, o.Name, o.Comment); err != nil {
-			return nil, fmt.Errorf("failed to add comment to table: %w", err)
-		}
+		dbActions = append(dbActions, NewCommentTableAction(conn, o.Name, o.Comment))
 	}
 
 	// Update the in-memory schema representation with the new table
 	o.updateSchema(s)
 
+	return &StartResult{Actions: dbActions}, nil
+}
+
+func (o *OpCreateTable) Complete(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationComplete(o)
+
+	// No-op
 	return nil, nil
 }
 
-func (o *OpCreateTable) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
-	// No-op
-	return nil
-}
+func (o *OpCreateTable) Rollback(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationRollback(o)
 
-func (o *OpCreateTable) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
-	_, err := conn.ExecContext(ctx, fmt.Sprintf("DROP TABLE IF EXISTS %s",
-		pq.QuoteIdentifier(o.Name)))
-	return err
+	return []DBAction{NewDropTableAction(conn, o.Name)}, nil
 }
 
 func (o *OpCreateTable) Validate(ctx context.Context, s *schema.Schema) error {
@@ -209,10 +204,14 @@ func (o *OpCreateTable) updateSchema(s *schema.Schema) *schema.Schema {
 			Unique:   col.Unique,
 			Nullable: col.Nullable,
 			Type:     col.Type,
+			Default:  col.Default,
 		}
 		if col.Pk {
 			primaryKeys = append(primaryKeys, col.Name)
 		}
+		if col.Comment != nil {
+			columns[col.Name].Comment = *col.Comment
+		}
 	}
 
 	uniqueConstraints := make(map[string]*schema.UniqueConstraint, 0)
