@@ -9,33 +9,42 @@ import (
 	"github.com/xataio/pgroll/pkg/schema"
 )
 
-var _ Operation = (*OpRawSQL)(nil)
+var (
+	_ Operation  = (*OpRawSQL)(nil)
+	_ Createable = (*OpRawSQL)(nil)
+)
+
+func (o *OpRawSQL) Start(ctx context.Context, l Logger, conn db.DB, s *schema.Schema) (*StartResult, error) {
+	l.LogOperationStart(o)
 
-func (o *OpRawSQL) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
 	if o.OnComplete {
 		return nil, nil
 	}
 
-	_, err := conn.ExecContext(ctx, o.Up)
-	return nil, err
+	dbActions := []DBAction{
+		NewRawSQLAction(conn, o.Up),
+	}
+	return &StartResult{Actions: dbActions}, nil
 }
 
-func (o *OpRawSQL) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
+func (o *OpRawSQL) Complete(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationComplete(o)
+
 	if !o.OnComplete {
-		return nil
+		return nil, nil
 	}
 
-	_, err := conn.ExecContext(ctx, o.Up)
-	return err
+	return []DBAction{NewRawSQLAction(conn, o.Up)}, nil
 }
 
-func (o *OpRawSQL) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
+func (o *OpRawSQL) Rollback(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationRollback(o)
+
 	if o.Down == "" {
-		return nil
+		return nil, nil
 	}
 
-	_, err := conn.ExecContext(ctx, o.Down)
-	return err
+	return []DBAction{NewRawSQLAction(conn, o.Down)}, nil
 }
 
 func (o *OpRawSQL) Validate(ctx context.Context, s *schema.Schema) error {
