@@ -21,7 +21,8 @@ func TestSetNotNull(t *testing.T) {
 			name: "set not null with default down sql",
 			migrations: []migrations.Migration{
 				{
-					Name: "01_add_table",
+					Name:          "01_add_table",
+					VersionSchema: "add_table",
 					Operations: migrations.Operations{
 						&migrations.OpCreateTable{
 							Name: "reviews",
@@ -51,7 +52,8 @@ func TestSetNotNull(t *testing.T) {
 					},
 				},
 				{
-					Name: "02_set_nullable",
+					Name:          "02_set_nullable",
+					VersionSchema: "set_nullable",
 					Operations: migrations.Operations{
 						&migrations.OpAlterColumn{
 							Table:    "reviews",
@@ -67,13 +69,13 @@ func TestSetNotNull(t *testing.T) {
 				ColumnMustExist(t, db, schema, "reviews", migrations.TemporaryName("review"))
 
 				// Inserting a NULL into the new `review` column should fail
-				MustNotInsert(t, db, schema, "02_set_nullable", "reviews", map[string]string{
+				MustNotInsert(t, db, schema, "set_nullable", "reviews", map[string]string{
 					"username": "alice",
 					"product":  "apple",
 				}, testutils.CheckViolationErrorCode)
 
 				// Inserting a non-NULL value into the new `review` column should succeed
-				MustInsert(t, db, schema, "02_set_nullable", "reviews", map[string]string{
+				MustInsert(t, db, schema, "set_nullable", "reviews", map[string]string{
 					"username": "alice",
 					"product":  "apple",
 					"review":   "amazing",
@@ -81,27 +83,27 @@ func TestSetNotNull(t *testing.T) {
 
 				// The value inserted into the new `review` column has been backfilled into the
 				// old `review` column.
-				rows := MustSelect(t, db, schema, "01_add_table", "reviews")
+				rows := MustSelect(t, db, schema, "add_table", "reviews")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "username": "alice", "product": "apple", "review": "amazing"},
 				}, rows)
 
 				// Inserting a NULL value into the old `review` column should succeed
-				MustInsert(t, db, schema, "01_add_table", "reviews", map[string]string{
+				MustInsert(t, db, schema, "add_table", "reviews", map[string]string{
 					"username": "bob",
 					"product":  "banana",
 				})
 
 				// The NULL value inserted into the old `review` column has been written into
 				// the new `review` column using the `up` SQL.
-				rows = MustSelect(t, db, schema, "02_set_nullable", "reviews")
+				rows = MustSelect(t, db, schema, "set_nullable", "reviews")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "username": "alice", "product": "apple", "review": "amazing"},
 					{"id": 3, "username": "bob", "product": "banana", "review": "banana is good"},
 				}, rows)
 
 				// Inserting a non-NULL value into the old `review` column should succeed
-				MustInsert(t, db, schema, "01_add_table", "reviews", map[string]string{
+				MustInsert(t, db, schema, "add_table", "reviews", map[string]string{
 					"username": "carl",
 					"product":  "carrot",
 					"review":   "crunchy",
@@ -109,7 +111,7 @@ func TestSetNotNull(t *testing.T) {
 
 				// The non-NULL value inserted into the old `review` column has been copied
 				// unchanged into the new `review` column.
-				rows = MustSelect(t, db, schema, "02_set_nullable", "reviews")
+				rows = MustSelect(t, db, schema, "set_nullable", "reviews")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "username": "alice", "product": "apple", "review": "amazing"},
 					{"id": 3, "username": "bob", "product": "banana", "review": "banana is good"},
@@ -125,7 +127,7 @@ func TestSetNotNull(t *testing.T) {
 				ColumnMustNotExist(t, db, schema, "reviews", migrations.TemporaryName("review"))
 
 				// Selecting from the `reviews` view should succeed.
-				rows := MustSelect(t, db, schema, "02_set_nullable", "reviews")
+				rows := MustSelect(t, db, schema, "set_nullable", "reviews")
 				assert.Equal(t, []map[string]any{
 					{"id": 2, "username": "alice", "product": "apple", "review": "amazing"},
 					{"id": 3, "username": "bob", "product": "banana", "review": "banana is good"},
@@ -133,7 +135,7 @@ func TestSetNotNull(t *testing.T) {
 				}, rows)
 
 				// Writing NULL reviews into the `review` column should fail.
-				MustNotInsert(t, db, schema, "02_set_nullable", "reviews", map[string]string{
+				MustNotInsert(t, db, schema, "set_nullable", "reviews", map[string]string{
 					"username": "daisy",
 					"product":  "durian",
 				}, testutils.NotNullViolationErrorCode)
