@@ -4,39 +4,36 @@ package migrations
 
 import (
 	"context"
-	"fmt"
 	"slices"
 	"strings"
 
-	"github.com/lib/pq"
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
 
 var _ Operation = (*OpSetReplicaIdentity)(nil)
 
-func (o *OpSetReplicaIdentity) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
-	// build the correct form of the `SET REPLICA IDENTITY` statement based on the`identity type
-	identitySQL := strings.ToUpper(o.Identity.Type)
-	if identitySQL == "INDEX" {
-		identitySQL = fmt.Sprintf("USING INDEX %s", pq.QuoteIdentifier(o.Identity.Index))
+func (o *OpSetReplicaIdentity) Start(ctx context.Context, l Logger, conn db.DB, s *schema.Schema) (*StartResult, error) {
+	l.LogOperationStart(o)
+
+	dbActions := []DBAction{
+		NewSetReplicaIdentityAction(conn, o.Table, o.Identity.Type, o.Identity.Index),
 	}
-
-	// set the replica identity on the underlying table
-	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s REPLICA IDENTITY %s",
-		pq.QuoteIdentifier(o.Table),
-		identitySQL))
-	return nil, err
+	return &StartResult{Actions: dbActions}, nil
 }
 
-func (o *OpSetReplicaIdentity) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
+func (o *OpSetReplicaIdentity) Complete(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationComplete(o)
+
 	// No-op
-	return nil
+	return nil, nil
 }
 
-func (o *OpSetReplicaIdentity) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
+func (o *OpSetReplicaIdentity) Rollback(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationRollback(o)
+
 	// No-op
-	return nil
+	return nil, nil
 }
 
 func (o *OpSetReplicaIdentity) Validate(ctx context.Context, s *schema.Schema) error {
