@@ -4,9 +4,8 @@ package migrations
 
 import (
 	"context"
-	"fmt"
 
-	"github.com/lib/pq"
+	"github.com/xataio/pgroll/pkg/backfill"
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -21,7 +20,9 @@ type OpSetUnique struct {
 
 var _ Operation = (*OpSetUnique)(nil)
 
-func (o *OpSetUnique) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
+func (o *OpSetUnique) Start(ctx context.Context, l Logger, conn db.DB, s *schema.Schema) (*StartResult, error) {
+	l.LogOperationStart(o)
+
 	table := s.GetTable(o.Table)
 	if table == nil {
 		return nil, TableDoesNotExistError{Name: o.Table}
@@ -31,24 +32,28 @@ func (o *OpSetUnique) Start(ctx context.Context, conn db.DB, latestSchema string
 		return nil, ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
 	}
 
-	return table, createUniqueIndexConcurrently(ctx, conn, s.Name, o.Name, table.Name, []string{column.Name})
+	dbActions := []DBAction{
+		NewCreateUniqueIndexConcurrentlyAction(conn, s.Name, o.Name, table.Name, column.Name),
+	}
+	return &StartResult{Actions: dbActions, BackfillTask: backfill.NewTask(table)}, nil
 }
 
-func (o *OpSetUnique) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
-	// Create a unique constraint using the unique index
-	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s ADD CONSTRAINT %s UNIQUE USING INDEX %s",
-		pq.QuoteIdentifier(o.Table),
-		pq.QuoteIdentifier(o.Name),
-		pq.QuoteIdentifier(o.Name)))
-	if err != nil {
-		return err
+func (o *OpSetUnique) Complete(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationComplete(o)
+
+	table := s.GetTable(o.Table)
+	if table == nil {
+		return nil, TableDoesNotExistError{Name: o.Table}
 	}
 
-	return err
+	// Create a unique constraint using the unique index
+	return []DBAction{NewAddConstraintUsingUniqueIndex(conn, o.Table, o.Name, o.Name)}, nil
 }
 
-func (o *OpSetUnique) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
-	return nil
+func (o *OpSetUnique) Rollback(l Logger, conn db.DB, s *schema.Schema) ([]DBAction, error) {
+	l.LogOperationRollback(o)
+
+	return nil, nil
 }
 
 func (o *OpSetUnique) Validate(ctx context.Context, s *schema.Schema) error {
