@@ -12,9 +12,6 @@ type options struct {
 	// disable pgroll version schemas creation and deletion
 	disableVersionSchemas bool
 
-	// disable creation of version schema for raw SQL migrations
-	noVersionSchemaForRawSQL bool
-
 	// additional entries to add to the search_path during migration execution
 	searchPath []string
 
@@ -22,6 +19,8 @@ type options struct {
 	skipValidation bool
 
 	migrationHooks MigrationHooks
+
+	verbose bool
 }
 
 // MigrationHooks defines hooks that can be set to be called at various points
@@ -53,18 +52,11 @@ func WithRole(role string) Option {
 	}
 }
 
-// WithDisableViewsManagement disables pgroll version schemas management
-// when passed, pgroll will not create or drop version schemas
-func WithDisableViewsManagement() Option {
+// WithVersionSchema enables or disables the creation of version schema for
+// migrations.
+func WithVersionSchema(enabled bool) Option {
 	return func(o *options) {
-		o.disableVersionSchemas = true
-	}
-}
-
-// WithNoVersionSchemaForRawSQL disables the creation of version schema for raw SQL migrations
-func WithNoVersionSchemaForRawSQL() Option {
-	return func(o *options) {
-		o.noVersionSchemaForRawSQL = true
+		o.disableVersionSchemas = !enabled
 	}
 }
 
@@ -93,3 +85,12 @@ func WithSkipValidation(skip bool) Option {
 		o.skipValidation = skip
 	}
 }
+
+// WithLogging enables verbose logging for the Roll instance
+func WithLogging(enabled bool) Option {
+	return func(o *options) {
+		if enabled {
+			o.verbose = enabled
+		}
+	}
+}
