@@ -7,6 +7,7 @@ import (
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
 	"github.com/xataio/pgroll/pkg/migrations"
 	"github.com/xataio/pgroll/pkg/sql2pgroll"
 	"github.com/xataio/pgroll/pkg/sql2pgroll/expect"
@@ -21,11 +22,11 @@ func TestConvertRenameStatements(t *testing.T) {
 	}{
 		{
 			sql:        "ALTER TABLE foo RENAME COLUMN a TO b",
-			expectedOp: expect.AlterColumnOp4,
+			expectedOp: expect.RenameColumnOp1,
 		},
 		{
 			sql:        "ALTER TABLE foo RENAME a TO b",
-			expectedOp: expect.AlterColumnOp4,
+			expectedOp: expect.RenameColumnOp1,
 		},
 		{
 			sql:        "ALTER TABLE foo RENAME TO bar",
